#!/bin/csh

##################################################################################################################################################
##################################################################################################################################################
##################################################################################################################################################
#                                                                                                                                                #
#          RUN THIS SCRIPT WITH 5 INPUTS DESCRIBING INFORMATION FROM LOCAL MACHINE (INPUTS DESCRIBED IN DETAIL BELOW)                            #
#                                                                                                                                                #
#  Example:                                                                                                                                      #
#      ./compile.csh 'jsmith@xxxxx.edu' 'Earthdata_user' 'Earthdata_passwd' '/home/jsmith/geocat_snow_ice_prod' '-L/home/jsmith/hdflib' 'intel'  #
#                                                                                                                                                #
##################################################################################################################################################
##################################################################################################################################################
##################################################################################################################################################

##################################################################################################################################################
##################################################################################################################################################
#
#   6 INPUTS EXPECTED BY SCRIPT
#
##################################################################################################################################################
##################################################################################################################################################

# INPUT 1
#Email address that will be used for FTP anonymous password when downloading data
#setenv EMAIL jsmith@xxxxxxxxxxxxx.edu
set EMAIL_in = $1
setenv EMAIL $EMAIL_in

# INPUT 2
#Earthdata login username
set EarthD_USER_in = $2
setenv EarthD_USER $EarthD_USER_in

# INPUT 3
#Earthdata login password
set EarthD_PASS_in = $3
setenv EarthD_PASS $EarthD_PASS_in

# INPUT 4
#directory containing the snow source code and modules
#setenv CODE /home/jsmith/geocat_snow_ice_prod
set CODE_in = $4
setenv CODE $CODE_in
#directory where final snow map will be written to
setenv DATA $CODE/data
if (! -e $DATA) mkdir $DATA

# INPUT 5
#directory where (-libmfhdf -libdf -libjpeg -libz) HDF libraries exist.
#setenv HDFLIBS -L/home/jsmith/hdflib
#format needed if HDF libraries are in multiple directories
#setenv HDFLIBS '-L/home/jsmith/hdflib1 -L/home/jsmith/hdflib2'
set HDFLIBS_in1 = `echo $5 | awk '{print $1}'`
set HDFLIBS_in2 = `echo $5 | awk '{print $2}'`
setenv HDFLIBS1 $HDFLIBS_in1
setenv HDFLIBS2 $HDFLIBS_in2

# INPUT 6
#Compiler to be used to compile code ('intel', 'gfortran', or 'aix')
#setenv COMPILER 'intel'
set COMPILER_in = $6
setenv COMPILER $COMPILER_in

setenv HOME ~/

#############################################################################################################################
#############################################################################################################################

echo ' '
echo '###################################################################'
echo 'Make sure the following information is correct'
echo 'If not, rerun compile.csh with correct inputs to fix'
echo '###################################################################'
echo ' '
echo 'Your email address is ' $EMAIL
echo ' '
echo 'Your Earthdata username is ' $EarthD_USER
echo ' '
echo 'Directory where source code and modules are located is ' $CODE
echo ' '
echo 'Directory where final snow product will be written is ' $DATA
echo ' '
echo 'HDF libraries are located in ' $HDFLIBS1 $HDFLIBS2
echo ' '
echo 'Compiler to be used for compilation is ' $COMPILER
echo '###################################################################'
echo ' '

if ($EMAIL == '') then
  echo 'Inputs not recognized, see README file for directions how to run compile.csh' 
  echo 'exit script'
  exit
endif

if (! -e $HOME) then
 echo 'Home directory does not exist, rerun compile.csh with proper input'
 exit
endif
if (! -e $CODE) then
 echo 'Source code directory does not exist, rerun compile.csh with proper input'
 exit
endif
if (! -e $DATA) then
 echo 'Final snow product data directory does not exist, rerun compile.csh with proper input'
 exit
endif

echo 'Create .netrc_snow file'
cat <<EOF>> $HOME/.netrc_snow
machine ftp.ssec.wisc.edu login anonymous password $EMAIL
machine sidads.colorado.edu login anonymous password $EMAIL
machine n0dps01u.ecs.nasa.gov login anonymous password $EMAIL
machine n4ftl01u.ecs.nasa.gov login anonymous password $EMAIL
machine n5eil01u.ecs.nsidc.org login anonymous password $EMAIL
machine urs.earthdata.nasa.gov login $EarthD_USER password $EarthD_PASS
EOF
chmod 600 $HOME/.netrc_snow

echo 'Create script to run code'
if (-e $CODE/temp1.csh) rm $CODE/temp1.csh
cat <<EOF1>> $CODE/temp1.csh
#! /bin/csh

setenv HOME $HOME
setenv CODE $CODE
setenv DATA $DATA

#need to create the .netrc file in C shell environment
if (-e $HOME/.netrc_snow) then
  if (-e $HOME/.netrc) then
   cp $HOME/.netrc $HOME/.netrc_bu
   rm $HOME/.netrc
  endif
  cp $HOME/.netrc_snow $HOME/.netrc
else
  echo '################ .netrc_snow file created at compilation is MISSING ###################'
endif

EOF1

if (-e $CODE/snow_ice_execute.csh) rm $CODE/snow_ice_execute.csh
cat temp1.csh temp2.csh > snow_ice_execute.csh
chmod 755 snow_ice_execute.csh
echo 'Script created'

echo ' '
echo 'Grab ims lat/lon data files from ftp server'
cd $CODE

ftp -v ftp.ssec.wisc.edu << XXX

passive
prompt
binary
cd /pub/geocat/geocat_snow_ice_prod
mget ims*_4km.dat.bz2

bye
XXX

#unzip files
echo ' '
echo ' Unzipping ims lat/lon files'
bunzip2 ims*_4km.dat.bz2
echo 'ims lat/lon files downloaded and unzipped'

echo ' '
echo 'Start compiling snow/ice code using the following compiler: ' $COMPILER

if ($COMPILER == 'intel' || $COMPILER == 'INTEL') then
echo 'use intel compiler'

#compile code run by script snow_ice_execute.csh
ifort -o julian2dy_pro julian2dy_pro.f90

#compile rest of snow/ice code
ifort -c constant.f90 equal_angle_grid.f90 julian_utils_mod.f90
if ("$HDFLIBS2" == '') then
 ifort -C snow_ice.f90 equal_angle_grid.o constant.o julian_utils_mod.o -o snow_ice $HDFLIBS1 -lmfhdf -ldf -ljpeg -lz
else
 ifort -C snow_ice.f90 equal_angle_grid.o constant.o julian_utils_mod.o -o snow_ice $HDFLIBS1 $HDFLIBS2 -lmfhdf -ldf -ljpeg -lz
endif
endif

if ($COMPILER == 'gfortran' || $COMPILER == 'GFORTRAN') then
echo 'use gfortran compiler'

#compile code run by script snow_ice_execute.csh
gfortran -o julian2dy_pro julian2dy_pro.f90

#compile rest of snow/ice code
gfortran -c constant.f90 equal_angle_grid.f90 julian_utils_mod.f90
if ("$HDFLIBS2" == '') then
 gfortran -C snow_ice.f90 equal_angle_grid.o constant.o julian_utils_mod.o -o snow_ice $HDFLIBS1 -lmfhdf -ldf -ljpeg -lz
else
 gfortran -C snow_ice.f90 equal_angle_grid.o constant.o julian_utils_mod.o -o snow_ice $HDFLIBS1 $HDFLIBS2 -lmfhdf -ldf -ljpeg -lz
endif
endif

if ($COMPILER == 'aix' || $COMPILER == 'AIX') then
echo 'use AIX compiler'

#compile code run by script snow_ice_execute.csh
xlf90 -O2 -o -qmaxmem=20000 -bmaxdata:0x80000000 -qfree=f90 -qsuffix=f=f90 julian2dy_pro julian2dy_pro.f90

#compile rest of snow/ice code
xlf90 -c -O2 -qmaxmem=20000 -bmaxdata:0x80000000 -qfree=f90 -qsuffix=f=f90 constant.f90 equal_angle_grid.f90 julian_utils_mod.f90
if ("$HDFLIBS2" == '') then
 xlf90 -O2 -qmaxmem=20000 -bmaxdata:0x80000000 -qfree=f90 -qsuffix=f=f90 -C snow_ice.f90 equal_angle_grid.o constant.o julian_utils_mod.o -o snow_ice $HDFLIBS1 -lmfhdf -ldf -ljpeg -lz -lsz
else
 xlf90 -O2 -qmaxmem=20000 -bmaxdata:0x80000000 -qfree=f90 -qsuffix=f=f90 -C snow_ice.f90 equal_angle_grid.o constant.o julian_utils_mod.o -o snow_ice $HDFLIBS1 $HDFLIBS2 -lmfhdf -ldf -ljpeg -lz -lsz
endif
endif

echo 'Completed compiling'
echo ' '
