! $Id: equal_angle_grid.f90,v 1.2 2012/04/24 14:50:09 justins Exp $
!------------------------------------------------------------------
!>This set of routines in this modules is used when the equal angle
!!options selected for the clavr-x gridcell
!!
!!this cell-ordering is the same as the PATMOS ordering
!!first cell is at 0 longitude (the prime meridian) and at
!!the south pole. Cells go east then north.
!-------------------------------------------------------------------
 module EQUAL_ANGLE_GRID
  use CONSTANTS
  implicit none
  private
  public:: SETUP_EQUAL_ANGLE,FIND_EQUAL_ANGLE_CELL
          
  !>Public variables with size of grid (returned by setup_equal_angle)
  integer, public, save:: num_lat,num_lon

  !>Public variables for sizing of grid and first point
  real(kind=real8), public, save:: dlat,dlon,first_lat,first_lon

  contains

!----------------------------------------------------------------------
!>setup the equal-area grid cells - this uses patmos convention
!!which is cell numbering starts at the south pole 0 longitude
!!and goes west and north
!!
!!this routine should be called before any other in this module
!!                                  
!! AUTHOR:\n                        
!!   Mike Pavolonis (NOAA/NESDIS)   
!!                                  
!! DATE:\n
!!   04/17/12
!!
!! PARAMETERS:\n
!!   dlat_in- Latitude spacing\n
!!   dlon_in - Longitude spacing\n
!!   num_lat and num_lon are size of respective equal angle grid
!------------------------------------------------------------------------

  subroutine SETUP_EQUAL_ANGLE(dlat_in,dlon_in)
    real(kind=real8), intent(in):: dlat_in, dlon_in

    dlat = dlat_in
    dlon = dlon_in
    
    num_lat = int(180.0/dlat)    !number of latitude cells
    num_lon = int(360.0/dlon)    !number of longitude cells

  end subroutine SETUP_EQUAL_ANGLE

!-------------------------------------------------------------------------
!>given a latitude (-90 to 90) and a longitude (-180 to 180) determine
!!which 1 deg equal area box this falls into.  These boxes are the
!!41258 boxes used in PATMOS mapping.
!!This code is mainly taken from routines written by V. Rao Anne
!!
!! AUTHOR:\n                        
!!   Mike Pavolonis (NOAA/NESDIS)   
!!                                  
!! DATE:\n
!!   04/17/12
!!  
!! PARAMETERS:\n
!!   latitude - Input latitude of point\n
!!   longitude - Input longitude of point\n
!!   ilat - Output latitude index of corresponding cell\n
!!   ilon - Output longitude index of corresponding cell\n
!!   ibox - Output cell cast into 41258 PATMOS mapping
!-------------------------------------------------------------------------

  subroutine FIND_EQUAL_ANGLE_CELL(latitude,longitude,ilat,ilon,ibox)
      real(kind=real4), intent(in):: latitude,longitude
      integer, intent(out):: ilat, ilon, ibox
      real(kind=real4):: rrlat,rrlon

!-------------------------------------------------------------------------
! determine equal area box
!-------------------------------------------------------------------------
   rrlat = latitude
   rrlon = longitude

   first_lat = -90.0
   first_lon = -180.0
  
   ilat = min(num_lat,int(abs(rrlat - first_lat)/dlat)+1)
   ilon = min(num_lon,int(abs(rrlon - first_lon)/dlon)+1)
   ibox = num_lon + ilon
   
  end subroutine FIND_EQUAL_ANGLE_CELL

end module EQUAL_ANGLE_GRID
