!------------------------------------------------------------------------------
!>    name:
!!      snow_ice
!!
!!--- function:
!!      To combine the NOAA/NESDIS IMS 4km snow/ice product with the 
!!      SSM/I 25 km snow/ice product to serve as snow mask for GEOCAT
!!
!!--- description:
!!      Creates a regular equal-angle 0.04 degree lat/lon grid and
!!      populates the northern hemisphere with the 4km IMS data and the
!!      southern hemisphere with the 25km SSM/I data. If no 4km IMS data
!!      then the global analysis will contain all 25km SSM/I data subsampled.
!!      Performs a fill of missing points by calculating the highest mode of
!!      the surrounding data.
!!
!!--- reference:
!!      Corey Calvert
!!      CIMSS - UW madison
!!
!!--- Calling sequence:
!!      ifort -c constant.f90 equal_angle_mpav.f90 julian_utils_mod.f90
!!      ifort -C snow_ice.f90 equal_angle_mpav.o constant.o julian_utils_mod.o -o
!!      snow_ice  -L/data1/hdf4 -lmfhdf -ldf -ljpeg -lz
!!      !ifort -o snow_ice snow_ice.f90 /data1/hdf4/libmfhdf.a
!!      /data1/hdf4/libdf.a /data1/hdf4/libjpeg.a /data1/hdf4/libz.a
!!      equal_angle_mpav.o constant.o julian_utils_mod.o
!!
!!      Program intended to run using the script snow_ice.csh which creates the
!!      proper input file.
!!      Running the program without the script can be done by manually changing
!!      snow_ice.input
!!
!!--- inputs:
!!      filenames of the IMS and SSM/I data files via input file created by a
!!      script
!!      lat/lon data for both datasets (NLLATLSB,NLLONLSB,SLLATLSB,SLLONLSB,
!!                                     imslat_4km.dat,imslon_4km.dat)
!!
!!--- outputs:
!!      HDF file containing the equal-angle (0.04 degree) snow_ice analysis
!!      containing no missing points. Lat/lon not included since they are 
!!      easily recreatable.
!!
!!--- dependencies:
!!     ! Run via a script (snow_ice.csh). Also see calling sequence and input
!!     lat/lon files
!!
!!--- restrictions:
!!
!!--- history:
!!      1/07 - Corey Calvert - Created
!------------------------------------------------------------------------------

program snow_ice

use EQUAL_ANGLE_GRID
use JULIAN_UTILS

implicit none

!---  declare variables
integer, parameter:: int1 = selected_int_kind(1)
integer, parameter:: int2 = selected_int_kind(3)
integer, parameter:: int4 = selected_int_kind(8)
integer, parameter:: real4 = selected_real_kind(6,37)
integer, parameter:: real8 = selected_real_kind(15,307)
integer :: i,j,m,n,mm,nn
integer :: ct,ckfiles
integer :: tx,ty,nx,ny
integer :: modeclr,modeice,modesnw
real(kind=real8) :: dellat,dellon,firstlat,firstlon
real :: tlat,tlon,ealat,ealon
real :: pi,deg2rad
real :: sclfct,offset
integer,parameter :: DFACC_READ=1,DFACC_CREATE=4
real,parameter :: lat_const = 111.320  !# of km in 1 deg latitude
real,parameter :: dlat25 = (25.0/lat_const)
real,parameter :: dlat4 = (4.0/lat_const)
integer :: box,yloc,xloc
integer, dimension(2,2) :: lat_corn,lon_corn
integer(kind=1),dimension(:,:),allocatable :: nthhem_ext,sthhem_ext
integer,dimension(:,:),allocatable :: nhsnow,shsnow,snow
integer(kind=1),dimension(:,:),allocatable :: tempsnow,easnow
integer(kind=4),dimension(:),allocatable :: ilat,ilon
real,dimension(:),allocatable :: shlat,shlon,nhlat,nhlon
real(kind=4),dimension(:,:),allocatable :: lat,lon
character(len=2) :: cyr,cmth,cdy
character(len=20) :: date_string, time_string, creation_date
!character(len=18) :: creation_date
integer :: yr,mth,dy,jdy
character(len=100) :: dir,datadir,fname
character(len=200) :: filename,latfname,lonfname,header,hdfname
integer :: sd_id,file_id,sds_id,istat
integer :: sfcreate,sfstart,sfend,dim_id,sfdimid
integer :: sfsdmname,sfsnatt,sfwdata,sfendacc
integer :: DFNT_CHAR8,DFNT_INT8,DFNT_FLOAT64,DFNT_FLOAT32
integer,dimension(2) :: dims,start,stride,edge

!********************************************************************

!--- Set up constants
pi = 4.0*atan(1.0)
deg2rad = pi/180.0


!--- Find out if 4km and 25km snow/ice products are available
read(*,*) ckfiles
print *, ''    !spacer
if (ckfiles == 0) print *, '25km snow/ice data not available, do not process'
if (ckfiles == 1) print *, 'no current 25km or 4km snow/ice data available, do not process'
if (ckfiles == 2) print *, 'only 25km snow/ice data currently available'
if (ckfiles == 3) print *, 'only 4km snow/ice data currently available, use previous 25 km data'
if (ckfiles == 4) print *, 'both 25km and 4km snow/ice data currently available'

!print *, 'start program'
read(*,'(a100)') dir
read(*,'(a100)') datadir
!print *, dir
!print *, datadir
!print *, ''    !spacer
!print *, 'code directory is ->', dir
!print *, 'data directory is ->', datadir
!print *, ''    !spacer

!********************************************************************
!--- Set up equal angle grid
!********************************************************************

!--- Set delta lat/lon for creating the equal angle grid
dellat = 0.04_real8
dellon = 0.04_real8
firstlat = -90.0_real8
firstlon = -180.0_real8

call SETUP_EQUAL_ANGLE(dellat,dellon)!,numlat_num_lon)

print *, ''    !spacer
print *, 'dimensions of equal angle grid =', num_lon,num_lat

allocate(tempsnow(num_lon,num_lat),stat=istat)
if (istat /= 0 ) then
 print *,'error allocating equal angle variable arrays'
 stop
endif

tempsnow = -99


!********************************************************************
!--- Read in 25km SSM/I snow_ice extent data
!********************************************************************
read(*,'(a100)') fname
!fname = 'NISE_SSMIF13_20061225.HDFEOS'
filename = trim(datadir)//trim(fname)
print *, ''    !spacer
print *, trim(filename)

!--- Grab date from filename so final hdf file is named properly
!         1         2         3         4         5         6         7
!1234567890123456789012345678901234567890123456789012345678901234567890
!NISE_SSMISF17_20070116.HDFEOS
read(fname(17:18),'(a2)') cyr
read(fname(19:20),'(a2)') cmth
read(fname(21:22),'(a2)') cdy
print *,  'date of 25km data: ','  yr = ',cyr, '  mth = ', cmth, '  dy = ', cdy

!--- want date of current

sd_id=sfstart(filename,DFACC_READ)

!--- call subroutine to get dimensions of HDF file variable
call getdim(sd_id,2,tx,ty)  !2 gives southern hemisphere snow/ice extent
print *, 'dimensions of 25km data =', tx,ty

!--- allocate variable array
allocate(nthhem_ext(tx,ty),sthhem_ext(tx,ty),nhsnow(tx,ty),shsnow(tx,ty), &
           ilat(tx*ty),ilon(tx*ty),shlat(tx*ty),shlon(tx*ty), &
           nhlat(tx*ty),nhlon(tx*ty), &
           stat=istat)

if (istat /= 0 ) then
 print *,'error allocating variable arrays'
 stop
endif

!--- read in data from hdf file
call readinthdf2d(sd_id,0,tx,ty,nthhem_ext)  !0 gives northern hemisphere snow/ice extent
call readinthdf2d(sd_id,2,tx,ty,sthhem_ext)  !2 gives southern hemisphere snow/ice extent

!Terminate access to the SD interface and close the file.
istat = sfend(sd_id)
if (istat /= 0 ) then
  print *, 'Cannot close HDF file - aborting'
  stop
endif

if (ckfiles >= 3) goto 811  !skip 25km northern hemisphere data, use 4km instead

!********************************************************************
!--- Read in lat/lon values from binary files and put 
!     northern hemisphere 25km data on eq ang grid
!********************************************************************
latfname = trim(dir)//'NLLATLSB'
open(unit=83,file=latfname,form='unformatted',status='old', &
      action='read',access='direct',recl=4*tx*ty)
read(unit=83,rec=1) ilat
close(83)
lonfname = trim(dir)//'NLLONLSB'
open(unit=84,file=lonfname,form='unformatted',status='old', &
      action='read',access='direct',recl=4*tx*ty)
read(unit=84,rec=1) ilon
close(84)
nhlat = (ilat/100000.0)
nhlon = (ilon/100000.0)

tlat = 0
tlon = 0
ct=1
do j=1,ty
 do i=1,tx
  nhsnow(i,j) = nthhem_ext(i,j)
  if (nthhem_ext(i,j) < 0) nhsnow(i,j) = nthhem_ext(i,j) + 256  !convert signed integer to unsigned (0-256)

!--- Convert naming scheme to blend products
!   0 -> no data
!   1 -> snow/ice free land/water
!   2 -> ice
!   3 -> snow
  if (nhsnow(i,j) < 50) nhsnow(i,j) = 1
  if (nhsnow(i,j) >= 50 .and. nhsnow(i,j) <= 100) nhsnow(i,j) = 2
  if (nhsnow(i,j) == 101) nhsnow(i,j) = 3
  if (nhsnow(i,j) == 102) nhsnow(i,j) = 0
  if (nhsnow(i,j) == 103) nhsnow(i,j) = 3
  if (nhsnow(i,j) >= 104 .and. nhsnow(i,j) <= 254) nhsnow(i,j) = 0
  if (nhsnow(i,j) == 255) nhsnow(i,j) = 1

!--- Put data on equal angle grid
  if (nhlat(ct) >= -90.0 .and. nhlat(ct) <= 90.0) then
   if (nhlon(ct) >= -180.0 .and. nhlon(ct) <= 180.0) then

!    write(10,*) nhlat(ct),nhlon(ct),nhsnow(i,j)

    !--- Find lower left and upper right corner of 25km box
    tlat = nhlat(ct)-(dlat25/2.0)
    tlon = nhlon(ct)-((25.0/(lat_const*cos(nhlat(ct)*deg2rad)))/2.0)
    call FIND_EQUAL_ANGLE_CELL(tlat,tlon,lat_corn(1,1),lon_corn(1,1),box)
    if (tlat < -90.0) lat_corn(1,1) = 1
    if (tlon < -180.0) lon_corn(1,1) = 1
!    print *, 'nhlat,nhlon =', nhlat(ct),nhlon(ct)
!    print *, 'tlat,tlon =', tlat,tlon

    tlat = nhlat(ct)+(dlat25/2.0)
    tlon = nhlon(ct)+((25.0/(lat_const*cos(nhlat(ct)*deg2rad)))/2.0)
    call FIND_EQUAL_ANGLE_CELL(tlat,tlon,lat_corn(2,2),lon_corn(2,2),box)
    if (tlat > 90.0) lat_corn(2,2) = num_lat
    if (tlon > 180.0) lon_corn(2,2) = num_lon
!    print *, 'tlat,tlon =', tlat,tlon
!    print *, 'lon_corn =', lon_corn
!    print *, 'lat_corn =', lat_corn

    !--- Populate any eq. ang. grid point in 25km box bounded by 4 corners
    do m=lon_corn(1,1),lon_corn(2,2)
     do n=lat_corn(1,1),lat_corn(2,2)
      tempsnow(m,n) = nhsnow(i,j)
     enddo
    enddo

   endif
  endif

  ct = ct + 1
 enddo
enddo

811 continue

!********************************************************************
!--- Read in lat/lon values from binary files and put 
!     southern hemisphere 25km data on eq ang grid
!********************************************************************
latfname = trim(dir)//'SLLATLSB'
open(unit=85,file=latfname,form='unformatted',status='old', &
      action='read',access='direct',recl=4*tx*ty)
read(unit=85,rec=1) ilat
close(85)
lonfname = trim(dir)//'SLLONLSB'
open(unit=86,file=lonfname,form='unformatted',status='old', &
      action='read',access='direct',recl=4*tx*ty)
read(unit=86,rec=1) ilon
close(86)
shlat = (ilat/100000.0)
shlon = (ilon/100000.0)

ct=1
do j=1,ty
 do i=1,tx
  shsnow(i,j) = sthhem_ext(i,j)
  if (sthhem_ext(i,j) < 0) shsnow(i,j) = sthhem_ext(i,j) + 256  !convert signed integer to unsigned (0-256)

!--- Convert naming scheme to blend products
!   0 -> no data
!   1 -> snow/ice free land/water
!   2 -> ice
!   3 -> snow
  if (shsnow(i,j) < 50) shsnow(i,j) = 1
  if (shsnow(i,j) >= 50 .and. shsnow(i,j) <= 100) shsnow(i,j) = 2
  if (shsnow(i,j) == 101) shsnow(i,j) = 3
  if (shsnow(i,j) == 102) shsnow(i,j) = 0
  if (shsnow(i,j) == 103) shsnow(i,j) = 3
  if (shsnow(i,j) >= 104 .and. shsnow(i,j) <= 254) shsnow(i,j) = 0
  if (shsnow(i,j) == 255) shsnow(i,j) = 1

!--- Put data on equal angle grid
  if (shlat(ct) >= -90.0 .and. shlat(ct) <= 90.0) then
   if (shlon(ct) >= -180.0 .and. shlon(ct) <= 180.0) then

!    if (shsnow(i,j) == 0 ) print *, shlat(ct),shlon(ct),shsnow(i,j)

!    write(10,*) shlat(ct),shlon(ct),shsnow(i,j)

    !--- Find lower left and upper right corner of 25km box
    tlat = shlat(ct)-(dlat25/2.0)
    tlon = shlon(ct)-((25.0/(lat_const*cos(shlat(ct)*deg2rad)))/2.0)
    call FIND_EQUAL_ANGLE_CELL(tlat,tlon,lat_corn(1,1),lon_corn(1,1),box)
    if (tlat < -90.0) lat_corn(1,1) = 1
    if (tlon < -180.0) lon_corn(1,1) = 1

    tlat = shlat(ct)+(dlat25/2.0)
    tlon = shlon(ct)+((25.0/(lat_const*cos(shlat(ct)*deg2rad)))/2.0)
    call FIND_EQUAL_ANGLE_CELL(tlat,tlon,lat_corn(2,2),lon_corn(2,2),box)
    if (tlat > 90.0) lat_corn(2,2) = num_lat
    if (tlon > 180.0) lon_corn(2,2) = num_lon

    !--- Populate any eq. ang. grid point in 25km box bounded by 4 corners
    do m=lon_corn(1,1),lon_corn(2,2)
     do n=lat_corn(1,1),lat_corn(2,2)
      tempsnow(m,n) = shsnow(i,j)
     enddo
    enddo

   endif
  endif

  ct = ct + 1
 enddo
enddo

deallocate(sthhem_ext,nthhem_ext,ilat,ilon,stat=istat)
if (istat /= 0 ) then
 print *,'error deallocating 25km variable arrays'
 stop
endif


if (ckfiles == 2) goto 812 !4km product not available, use 25km only
!********************************************************************
!--- Read in 4km data
!********************************************************************
read(*,'(a100)') fname
!fname = 'ims2006359_4km.asc'
filename = trim(datadir)//trim(fname)
print *, ''    !spacer
print *, trim(filename)

nx = 6144
ny = 6144
print *, 'dimensions of 4km data =', nx,ny

!--- Grab date from filename so final hdf file is named properly
!         1         2         3         4         5         6         7
!1234567890123456789012345678901234567890123456789012345678901234567890
!ims2007016_4km.asc
read(fname(6:7),'(i2)') yr
read(fname(8:10),'(i3)') jdy
!print *, 'yr =', yr, 'jday =', jdy

call julian2yrmthdy(yr,jdy,mth,dy) !conovert julian day to mth and dy

!--- Convert integer into character strings to be used in hdf filename
if (yr < 10) then
 write(cyr(1:2),'(i1)') yr
 cyr = '0'//cyr
else
 write(cyr(1:2),'(i2)') yr 
endif
if (mth < 10) then
 write(cmth(1:2),'(i1)') mth
 cmth = '0'//cmth
else
 write(cmth(1:2),'(i2)') mth
endif
if (dy < 10) then
 write(cdy(1:2),'(i1)') dy
 cdy = '0'//cdy
else
 write(cdy(1:2),'(i2)') dy
endif
print *,  'date of 4km data: ','  yr = ',cyr, '  mth = ', cmth, '  dy = ', cdy
!********************************************************************

allocate(snow(nx,ny),lat(nx,ny),lon(nx,ny),stat=istat)
if (istat /= 0 ) then
 print *,'error allocating 4km variable arrays'
 stop
endif

open(87,file=filename,status='old',action='read')

!--- Read header lines from ascii file
do i=1,30
 read(87,'(a)') header
! print *, header
enddo

!--- Read in 4km snow/ice data
!   read(87,'(37748736(i1))') snow
 do i=1,nx
   read(87,'(6144(i1))') snow(:,i)
 enddo
close(87)

!********************************************************************
!--- Read in 4km lat/lon data
!********************************************************************
fname = 'imslat_4km.dat'
filename = trim(dir)//trim(fname)
!print *, trim(filename)

open(88,file=filename,status='old',action='read')
!read(88,'(37748736(f10.5))') lat
do j=1,ny
 do i=1,nx
  read(88,'(f11.5)') lat(i,j)

!--- Match to same naming scheme as the 25km product for merging
  if (snow(i,j) == 2) snow(i,j) = 1
  if (snow(i,j) == 3) snow(i,j) = 2
  if (snow(i,j) == 4) snow(i,j) = 3

 enddo
enddo
close(88)

!print *, lat(1,:)

fname = 'imslon_4km.dat'
filename = trim(dir)//trim(fname)
!print *, trim(filename)

open(89,file=filename,status='old',action='read')
!  read(89,'(37748736(f10.5))') lon
do j=1,ny
 do i=1,nx
  read(89,'(f10.5)') lon(i,j)
  if (lon(i,j) > 180.0 ) lon(i,j) = lon(i,j) - 360.0  !Convert lons from 0-360 to -180-180

  if (lat(i,j) > 0.0 ) then

!   write(20,*) lat(i,j),lon(i,j),snow(i,j)

    !--- Find lower left and upper right corner of 4km box
    tlat = lat(i,j)-(dlat4/2.0)
    tlon = lon(i,j)-((4.0/(lat_const*cos(lat(i,j)*deg2rad)))/2.0)
    call FIND_EQUAL_ANGLE_CELL(tlat,tlon,lat_corn(1,1),lon_corn(1,1),box)
    if (tlat < -90.0) lat_corn(1,1) = 1
    if (tlon < -180.0) lon_corn(1,1) = 1
!    print *, 'lat,lon =', lat(i,j),lon(i,j)
!    print *, 'tlat,tlon =', tlat,tlon

    tlat = lat(i,j)+(dlat4/2.0)
    tlon = lon(i,j)+((4.0/(lat_const*cos(lat(i,j)*deg2rad)))/2.0)
    call FIND_EQUAL_ANGLE_CELL(tlat,tlon,lat_corn(2,2),lon_corn(2,2),box)
    if (tlat > 90.0) lat_corn(2,2) = num_lat
    if (tlon > 180.0) lon_corn(2,2) = num_lon
!    print *, 'tlat,tlon =', tlat,tlon
!    print *, 'lon_corn =', lon_corn
!    print *, 'lat_corn =', lat_corn

    !--- Populate any eq. ang. grid point in 4km box bounded by 4 corners
    do m=lon_corn(1,1),lon_corn(2,2)
     do n=lat_corn(1,1),lat_corn(2,2)
      tempsnow(m,n) = snow(i,j)
     enddo
    enddo

  endif

 enddo
enddo
close(89)

deallocate(snow,lat,lon,stat=istat)
if (istat /= 0 ) then
 print *,'error deallocating 4km variable arrays'
 stop
endif

812 continue

deallocate(nhsnow,shsnow,shlat,shlon,nhlat,nhlon,stat=istat)
if (istat /= 0 ) then
 print *,'error deallocating remaining initial variable arrays'
 stop
endif
!********************************************************************

allocate(easnow(num_lon,num_lat),stat=istat)
if (istat /= 0 ) then
 print *,'error allocating equal angle variable arrays'
 stop
endif

!--- copy data so mode can be taken properly (i.e., w/out missing values continuously filled)
easnow = tempsnow

do i=1,num_lon
 do j=1,num_lat
   ealon = first_lon + (float(i-1)*dlon)
   ealat = first_lat + (float(j-1)*dlat)

   !--- calc mode in concentric rings surrounding gridpoints to fill in missing values
   if (tempsnow(i,j) == -99 .or. tempsnow(i,j) == 0) then
!   if (tempsnow(i,j) == 0) then

!    print *, 'i,j =', i,j
    modeclr = 0
    modeice = 0
    modesnw = 0
    do m=1,100
!     print *, 'm =', m
     mm = -1*m
     nn = -1*m
     do n=1,(2*m+1)
!       print *, (i+mm),(j+nn)
      if ((i+mm) >= 1.and.(i+mm) <= num_lon.and. &
          (j+nn) >= 1.and.(j+nn) <= num_lat) then
!       print *, 'mm,nn =', mm,nn
       if (tempsnow(i+mm,j+nn) == 1) modeclr = modeclr+1
       if (tempsnow(i+mm,j+nn) == 2) modeice = modeice+1
       if (tempsnow(i+mm,j+nn) == 3) modesnw = modesnw+1
      endif
      nn = nn + 1
     enddo
     mm = (-1*m)+1
     nn = -1*m
     do n=2,(2*m+1)
!       print *, (i+mm),(j+nn)
      if ((i+mm) >= 1.and.(i+mm) <= num_lon.and. &
          (j+nn) >= 1.and.(j+nn) <= num_lat) then
!       print *, 'mm,nn =', mm,nn
       if (tempsnow(i+mm,j+nn) == 1) modeclr = modeclr+1
       if (tempsnow(i+mm,j+nn) == 2) modeice = modeice+1
       if (tempsnow(i+mm,j+nn) == 3) modesnw = modesnw+1
      endif
      mm = mm + 1
     enddo
     mm = (-1*m)+1
     nn = m
     do n=2,(2*m+1)
!       print *, (i+mm),(j+nn)
      if ((i+mm) >= 1.and.(i+mm) <= num_lon.and. &
          (j+nn) >= 1.and.(j+nn) <= num_lat) then
!       print *, 'mm,nn =', mm,nn
       if (tempsnow(i+mm,j+nn) == 1) modeclr = modeclr+1
       if (tempsnow(i+mm,j+nn) == 2) modeice = modeice+1
       if (tempsnow(i+mm,j+nn) == 3) modesnw = modesnw+1
      endif
      mm = mm + 1
     enddo
     mm = m
     nn = (-1*m)+1
     do n=2,(2*m)
!       print *, (i+mm),(j+nn)
      if ((i+mm) >= 1.and.(i+mm) <= num_lon.and. &
          (j+nn) >= 1.and.(j+nn) <= num_lat) then
!       print *, 'mm,nn =', mm,nn
       if (tempsnow(i+mm,j+nn) == 1) modeclr = modeclr+1
       if (tempsnow(i+mm,j+nn) == 2) modeice = modeice+1
       if (tempsnow(i+mm,j+nn) == 3) modesnw = modesnw+1
      endif
      nn = nn + 1
     enddo
!     print *, 'mode clr/ice/snw =', modeclr,modeice,modesnw
     if (modeclr > 0 .and. modeclr > modeice .and. modeclr > modesnw) then
       easnow(i,j) = 1
       exit
     endif
     if (modeice > 0 .and. modeice > modeclr .and. modeice > modesnw) then
       easnow(i,j) = 2
       exit
     endif
     if (modesnw > 0 .and. modesnw > modeclr .and. modesnw > modeice) then
       easnow(i,j) = 3
       exit
     endif
    enddo

!   stop

   endif

    !--- write data on equal angle grid out to an ascii file
!    write(30,*) ealat,ealon,easnow(i,j)

 enddo
enddo

print *, 'min snow =', minval(easnow)

!********************************************************************
!--- write data out to HDF file
!********************************************************************

call date_and_time(date=date_string,time=time_string)
creation_date = date_string(5:6)//"/"//date_string(7:8)//"/"//date_string(1:4)//"-"// &
                time_string(1:2)//":"//time_string(3:4)//":"//time_string(5:6)

!print *, 'creation date =', creation_date
!print *, 'dellat/lon =', dellat,dellon

hdfname = 'snow_map_4km_'//cyr//cmth//cdy//'.hdf'
filename = trim(datadir)//trim(hdfname)
print *, ''    !spacer
print *, 'the following hdf file was created:'
print *, trim(filename)

file_id=sfstart(filename,DFACC_CREATE)
if (file_id == -1) then
  print *, 'Cannot create HDF file - aborting'
  stop
endif

dims(1) = num_lon  !x-dimension
dims(2) = num_lat  !y-dimension

!--- declare HDF data types
DFNT_CHAR8 = 4     !8-bit character type
DFNT_INT8 = 20     !8-bit integer type
DFNT_FLOAT32 = 5   !32-bit floating-point type
DFNT_FLOAT64 = 6   !64-bit floating-point type
sclfct = 1.0
offset = 0.0

!write global attributes
istat = sfsnatt(file_id,'creation_date',DFNT_CHAR8,20,creation_date)
istat = sfsnatt(file_id,'dlat',DFNT_FLOAT64,1,dellat)
istat = sfsnatt(file_id,'dlon',DFNT_FLOAT64,1,dellon)
istat = sfsnatt(file_id,'first_lat',DFNT_FLOAT64,1,firstlat)
istat = sfsnatt(file_id,'first_lon',DFNT_FLOAT64,1,firstlon)

sds_id = sfcreate(file_id,'snow_ice_cover',DFNT_INT8,2,dims)

dim_id = sfdimid(sds_id,0)
istat = sfsdmname(dim_id,'elements')

dim_id = sfdimid(sds_id,1)
istat = sfsdmname(dim_id,'lines')

istat = sfsnatt(sds_id,'reference',DFNT_CHAR8,35, &
           '[1 = land/water, 2 = ice, 3 = snow]')
istat = sfsnatt(sds_id,'units',DFNT_CHAR8,4,'none')
istat = sfsnatt(sds_id,'scaling_method',DFNT_INT8,1,1)
istat = sfsnatt(sds_id,'scale_factor',DFNT_FLOAT32,1,sclfct)
istat = sfsnatt(sds_id,'add_offset',DFNT_FLOAT32,1,offset)

start  = 0
stride = 1
edge(1) = num_lon
edge(2) = num_lat 
!print *, sds_id,start,stride,edge
!print *, easnow
istat = sfwdata(sds_id,start,stride,edge,easnow)
!print *, 'istat', istat

istat = sfendacc(sds_id)

!Terminate access to the SD interface and close the file.
istat = sfend(file_id)
if (istat /= 0 ) then
  print *, 'Cannot close HDF file - aborting'
  stop
endif

!********************************************************************

deallocate(tempsnow,easnow,stat=istat)
if (istat /= 0 ) then
 print *,'error deallocating final variable arrays'
 stop
endif

!********************************************************************

close(10)
close(20)
close(30)
end program snow_ice




!********************************************************************
!--- Subroutine section
!********************************************************************

!--------------------------------------------------------------------
!>This subourinte generate the x-dim and y-dim of the input HDF file
!!                                  
!! AUTHOR:\n                        
!!   Mike Pavolonis (NOAA/NESDIS)   
!!                                  
!! DATE:\n
!!   04/17/12
!!
!! PARAMETERS:\n
!!   sd_id- HDF file ID\n
!!   sds_index - Index for needed variable\n
!!   nx - output x-dim size
!!   ny - output y-dim size
!--------------------------------------------------------------------

SUBROUTINE getdim(sd_id,sds_index,nx,ny)

implicit none

!This subroutine generates the x- and y-dimensions of the HDF
!file.

integer, intent(in) :: sd_id,sds_index
integer, intent(out) :: nx, ny

!Declare the local variables.

integer :: sds_id,istat,n_datasets, &
           n_file_attr,sf_info,rank,num_type, &
           attributes,sf_ginfo
character(30) :: name
integer, dimension(2) :: dim_sizes

!Internal function declaration from linked files

integer :: sfselect,sfendacc
integer :: sffattr,sfrdata
integer :: sfgainfo,sfrnatt,sffinfo,sfginfo

!********************************************************************

!write(*,*) 'file ID is', sd_id

sf_info = sffinfo(sd_id, n_datasets, n_file_attr)

sds_id = sfselect(sd_id, sds_index)

sf_ginfo = sfginfo(sds_id,name,rank,dim_sizes,num_type, &
                           attributes)

nx = dim_sizes(1)
ny = dim_sizes(2)

!write(*,*) name,'dim_size',dim_sizes(1),nx,dim_sizes(2),ny

return
end subroutine getdim
!********************************************************************

!--------------------------------------------------------------------
!>!This subroutine reads in the scientific data from HDF files
!!using HDF4.  The inputs are the sds_index number, which gives
!!the data set that is to be obtained; the hdf_filename, which
!!gives the name of the HDF file to be opened and read.  It
!!returns the 2-D array of the data set described by the
!!provided sds_index number and the values of nx and ny, which
!!are the dimensions of the array.
!!                                  
!! AUTHOR:\n                        
!!   Mike Pavolonis (NOAA/NESDIS)   
!!                                  
!! DATE:\n
!!   04/17/12
!!
!! PARAMETERS:\n
!!   sd_id- HDF file ID\n
!!   sds_index - Index for needed variable\n
!!   nx - x-dim size
!!   ny - y-dim size
!!   data_2d - data read from HDF file
!--------------------------------------------------------------------

SUBROUTINE readinthdf2d(sd_id,sds_index,nx,ny,data_2d)

implicit none

!This subroutine reads in the scientific data from HDF files
!using HDF4.  The inputs are the sds_index number, which gives
!the data set that is to be obtained; the hdf_filename, which
!gives the name of the HDF file to be opened and read.  It
!returns the 2-D array of the data set described by the
!provided sds_index number and the values of nx and ny, which
!are the dimensions of the array.

!Declare calling parameters

integer, intent(in) :: sd_id,sds_index,nx,ny
integer(kind=1), dimension(nx,ny), intent(out) :: data_2d
!integer, dimension(nx,ny), intent(out) :: data_2d

!Declare the local variables.

integer :: sds_id,istat,n_datasets, &
           n_file_attr,sf_info,rank,num_type, &
           attributes,sf_ginfo
integer :: i,j
character(24) :: name
integer, dimension(2) :: start,edges,stride,dim_sizes

!Internal function declaration from linked files

integer :: sfselect,sfrdata,sfendacc,sffattr
integer :: sfgainfo,sfrnatt,sffinfo,sfginfo

!********************************************************************

sf_info = sffinfo(sd_id, n_datasets, n_file_attr)

!write(*,*) '# data sets:',n_datasets, '# attributes', &
!            n_file_attr

sds_id = sfselect(sd_id, sds_index)

!write(*,*) 'sds_id is',sds_id,'sds_index is',sds_index

sf_ginfo = sfginfo(sds_id,name,rank,dim_sizes,num_type, &
                           attributes)

!write(*,*) 'name',name
!write(*,*) 'rank',rank
!write(*,*) 'dim_size',dim_sizes(1),nx,dim_sizes(2),ny
!write(*,*) 'num_type',num_type
!write(*,*) 'attributes',attributes

!Set element of the array start to 0, elements of the array
!edges to the dimensions of the SDS data set, and elements of
!the array stride to 1 to read the entire data field without
!skipping data.  The variable data_2d is a dummy variable
!that passes all the data to be read into an array to the
!main executable code.

start(1) = 0
start(2) = 0
stride(1) = 1
stride(2) = 1
edges(1) = dim_sizes(1)
edges(2) = dim_sizes(2)

istat = sfrdata(sds_id, start, stride, edges, data_2d)
!print *, data_2d(1,1),data_2d(7,420)

!write(*,*) dim_sizes(1), dim_sizes(2)
!write(*,*) edges(1), edges(2)

!Terminate access to the data set.
istat = sfendacc(sds_id)

!write(*,*) "nx",nx,"ny",ny

return
end subroutine readinthdf2d
!********************************************************************
